package main

import (
	"context"
	"fmt"
	"log"
	"query-service/internal/aql"
	"query-service/internal/errorhandler"
	"query-service/internal/messagequeue"
	"query-service/internal/redisclient"

	"query-service/internal/request"

	"github.com/streadway/amqp"
	"github.com/thijsheijden/alice"
)

var producer alice.Producer

func main() {
	exchangeID := "query-requests"
	routingKey := "aql-user-request"

	broker := messagequeue.Create()
	messagequeue.StartConsumer(broker, &exchangeID, &routingKey, onMessageReceived)

	producer = messagequeue.StartProducer(broker)

	redisclient.Start()

	select {}
}

func onMessageReceived(msg amqp.Delivery) {
	// Retrieve JSON formatted string payload from msg

	// Bypass MSQ
	// jsonQuery, _ := ioutil.ReadFile("./internal/data/jsonQuery.json")
	aqlQuery, err := aql.ConvertJSONToAQL(&msg.Body)

	// Convert the json byte msg to an aql query string
	//aqlQuery, err := aql.ConvertJSONToAQL(&msg.Body)
	if err != nil {
		errorhandler.LogError(err, "failed to parse incoming msg to AQL") // TODO: don't panic on error, send error message to client instead
		return
	}
	fmt.Println("Query: " + *aqlQuery)

	// Get the queueID for this sessionID
	sessionID, ok := msg.Headers["sessionID"].(string)
	if !ok {
		log.Println("No sessionID passed in message")
		return
	}

	// Get queueID for this client
	queueID := redisclient.Conn.Get(context.Background(), sessionID).Val()
	log.Println(queueID)

	// producer.PublishMessage(aqlQuery, , &amqp.Table{})

	// TODO : Generate databse seperatly
	// execute and retrieve result
	// convert result to general (node-link (?)) format
	result, err := request.SendAQLQuery(*aqlQuery)
	if err != nil {
		return // TODO: Send message in queue notifying of error
	}

	// publish converted result
	headers := amqp.Table{}
	headers["sessionID"] = sessionID
	headers["type"] = "schemaResult"
	log.Println("publishing message")
	producer.PublishMessage(result, &queueID, &headers)
	msg.Ack(true)
}
